using Microsoft.AspNetCore.Mvc;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace Ewide.Core.Service
{
    public interface ISysMenuService
    {
        Task AddMenu(AddMenuInput input);
        Task<List<AntDesignTreeNode>> ChangeAppMenu(ChangeAppMenuInput input);
        Task DeleteMenu(DeleteMenuInput input);
        Task<List<AntDesignTreeNode>> GetLoginMenusAntDesign(string userId, string appCode);
        Task<List<string>> GetLoginPermissionList(string userId);
        Task<dynamic> GetMenu(QueryMenuInput input);
        Task<dynamic> GetMenuList([FromQuery] MenuInput input);
        Task<dynamic> GetMenuTree([FromQuery] MenuInput input);
        Task<List<string>> GetUserMenuAppCodeList(string userId);
        Task<bool> HasMenu(string appCode);
        Task<dynamic> TreeForGrant([FromQuery] MenuInput input);
        Task UpdateMenu(UpdateMenuInput input);
    }
}